package provide app-hindex 1.0

package require Tk
package require Tcl
package require hindex

global basedir 

# set up directory to read from differently for starkit/pack (second
# case) than in the case of an OS X .app package (first case)
if {[catch {package present starkit}]} {
    set basedir [file dirname $argv0]
} else {
    set basedir [file join $starkit::topdir lib app-hindex]
}

proc screen {} {

global indfile parfile outfile indEntry parEntry outEntry main 

wm title . "hindex"
wm protocol . WM_DELETE_WINDOW exit
DoMenus

frame .firstline -borderwidth 5
pack .firstline -fill x

label .firstline.indfileLabel -text "Individual file:" -width 15
set indEntry [ entry .firstline.indfileEntry -width 50 -relief sunken -textvariable indfile ]
button .firstline.browse -text "Browse..." -command {OpenFile $indEntry}

pack .firstline.indfileLabel .firstline.indfileEntry .firstline.browse \
    -side left -padx 5 -pady 5 -anchor n

frame .secondline -borderwidth 5
pack .secondline -fill x

label .secondline.parfileLabel -text "Parental file:" -width 15
set parEntry [ entry .secondline.parfileEntry -width 50 -relief sunken -textvariable parfile ]
button .secondline.browse -text "Browse..." -command {OpenFile $parEntry}
pack .secondline.parfileLabel .secondline.parfileEntry .secondline.browse \
    -side left -padx 5 -pady 5 -anchor n

frame .thirdline -borderwidth 5
pack .thirdline -fill x

label .thirdline.outfileLabel -text "Output file:" -width 15
set outEntry [ entry .thirdline.outfileEntry -width 50 -relief sunken -textvariable outfile ]
button .thirdline.newfile -text "New..." -command {NewFile $outEntry}
pack .thirdline.outfileLabel .thirdline.outfileEntry .thirdline.newfile -side left -padx 5 -pady 5 -anchor n


labelframe .bottom -text Output:

set main [text .bottom.main  -width 85 -height 30 -background gray -relief raised \
	      -setgrid true -bd 2  -yscrollcommand {.bottom.scroll set}]
scrollbar .bottom.scroll -command {.bottom.main yview}

pack .bottom.scroll -side right -fill y
pack .bottom.main -side left -fill both

pack .bottom -fill both -expand true

frame .buttons -borderwidth 5
button .buttons.run -text "Do calculations" -command RunHindex
button .buttons.clearall -text "Clear all" -command ClearAll
button .buttons.exit -text "Exit" -command exit

pack .buttons -fill x
pack .buttons.run .buttons.clearall -side left -padx 5 -pady 5 -anchor n
pack .buttons.exit -side right -padx 5 -pady 5 -anchor n
focus .firstline.indfileEntry
}


proc ClearAll {} {
    global main indEntry parEntry outEntry
    $main delete 1.0 end
    $indEntry delete 0 end
    $parEntry delete 0 end
    $outEntry delete 0 end
}

proc OpenFile {fieldname} {
    global outfile 
    $fieldname delete 0 end
    set chosenname [tk_getOpenFile]
    if {$chosenname != "" } {
	$fieldname insert 1 "$chosenname"
    }
    if {$outfile == ""} {
	set temp [file dirname $chosenname]
	set outfile "$temp/hindex_output.txt"
    }
}

proc NewFile {fieldname} {
    set chosenname [tk_getSaveFile -title "Save Output" -initialfile "hindex_output.txt" ]
    if {$chosenname != "" } {
	$fieldname delete 0 end 
	$fieldname insert 1 "$chosenname"
    }
}

proc RunHindex {} {
    global parfile indfile outfile main outEntry

    # if exists test for outfile
    if [ file exists $outfile ] {
	set choice [tk_messageBox -type yesnocancel -default no -icon question \
			-message "Output file of selected name already exists.  Overwrite existing file?" ]
    } else {
	set choice yes
    }

    if { $choice == yes } {
	if { [hindex $parfile $indfile $outfile] == 1 } {
	    tk_messageBox -message "The hybrid index routine returned an unknown error." -icon error
	} else {
	    displayOutput $outfile
	} 
    } elseif { $choice == no } {
	NewFile $outEntry 
	if { [hindex $parfile $indfile $outfile] == 1 } {
	    tk_messageBox -message "The hybrid index routine returned an unknown error." -icon error
	} else {
	    displayOutput $outfile
	} 
    } 
}

proc displayOutput {workingfile} {
    global main
    # clear contents of output field if it already contains some text
    set result [ $main compare insert > 1.0 ]
    if {$result} {
	$main delete 1.0 end 
    } 
    if [catch {open $workingfile r} outdata] {
	tk_messageBox -type ok -icon error -message "Could not open and display $workingfile"
    } else {
	foreach line [split [read $outdata] \n] {
	    $main insert insert $line\n
	}
	close $outdata
    }
}

proc DoMenus {} {
   . configure -menu [menu .m -tearoff 0]
   .m add cascade -menu [menu .m.file -tearoff 0] -label "File" -underline 0
   .m add cascade -menu [menu .m.info -tearoff 0] -label "Info" -underline 0

   .m.file add command -label "Do calculations" -under 0 -command RunHindex
   .m.file add separator
   .m.file add command -label Exit -under 0 -command exit

   .m.info add command -label Help  -under 0 -command Help
   .m.info add separator
   .m.info add command -label "About hindex" -under 0 -command About
   .m.info add command -label "Licenses" -under 0 -command gpl
}

proc gpl {} {
    global basedir
    set license $basedir/LICENSE
    displayOutput $license
}

proc Help {} {
    global basedir
    set readme $basedir/README.txt
    displayOutput $readme
}

proc About {} {
    tk_messageBox -type ok -icon info -message "hindex (Version 1.41)\n\nCopyright (C) 2003 -- Alex Buerkle\nPlease read the Help and License information." 
}

screen
